/******** <lalVerbatim file="FCCTonditionDataCV"> ******** 
Author: Wen, L.; Edlund, Jeffrey
$Id: FCTConditionData.c,v 1.1 2001/12/13 00:32:28 teviet Exp $ 
********* </lalVerbatim> ********/ 

#include <FCTGeneral.h>
#include <FCTConditionData.h>
/* #include <FCTHierSupport.h> */

NRCSID( FCTCONDITIONDATAC, "$Id: FCTConditionData.c,v 1.1 2001/12/13 00:32:28 teviet Exp $");

void LALConditionData( 
		      LALStatus *status, 
		      LALSearchInput *inout, 
		      void *searchParams, 
		      LALMPIParams          *mpiParams
		      ) 
{ 
  /* This Function is just a wrapper for HierConditionData */
  FCTSearchParams *mySearchParams;
  
  INITSTATUS( status, "LALConditionData", FCTCONDITIONDATAC ); 
  ATTATCHSTATUSPTR (status);
  
#ifdef FCT_SO_DEBUG
  fprintf(stdout," in dataCondition()\n");
  fflush( stdout );
#endif
  
  /* Casting void pointer searchParams into FCTSearchParams type. */
  mySearchParams = (FCTSearchParams *) searchParams;

  /* Call our data conditioning subroutine. */
  HierConditionData(status->statusPtr, inout, mySearchParams);
  CHECKSTATUSPTR (status);
  
#ifdef FCT_SO_DEBUG  
  fprintf(stdout," out of ConditionData()\n");
  fflush( stdout );
#endif
  
  DETATCHSTATUSPTR (status);
  RETURN( status ); 
} 

  
void HierConditionData(
		       LALStatus *status,
		       LALSearchInput *inout,
		       FCTSearchParams *mySearchParams
		       )
{
  void changeFCTPlan(LALStatus *status, FCTSearchParams * mySearchParams, INT4 dataLength);  
  INT4                          j;

  /* number of data sets in LALSearchInput */
  INT4                          numSequences = 3;

  /* The indices for the data sets in the LALSearchInput structure. */
  INT4                          dmroIndex  = 0;
  INT4                          specIndex  = 1;
  INT4                          respIndex  = 2;

  /* These variables are confusing and have meaningless names, but they are used a lot
     in the data conditioning stuff. */
  REAL4                         *dummyI2   = NULL; 
  REAL4                         *dummyR4   = NULL;
  COMPLEX8                      *dummyC8   = NULL;

  REAL8                         distNorm = 0.0;

  /* DFT */
  COMPLEX8Vector* tmpIn;  /*  timeseries data  */
  COMPLEX8Vector* tmpOut; /*  frequencyseries data  */
  ComplexFFTPlan* plan;
  FCTMomentType freqMoment;

 INT4  respShape=1; /* check, 1 for U shape response, -1 for n shape response */
  REAL4 dt=0.0, temp;  /* use if | R(f)|^2 is out of range */
  const REAL4   fac=1.0e-18;  /* check  */
  
  
  /* Variables used to make time slices and freq cutoffs. */
  INT4 startIndex, stopIndex; 
  /*REMOVE REAL4 inputDataStart, inputDataStop;*/
  INT4                          timeDataSliceLength;
  /* INT4                          freqDataSliceLength;*/

  INITSTATUS( status, "HierConditionData", FCTCONDITIONDATAC ); 
  ATTATCHSTATUSPTR (status);

#ifdef FCT_SO_DEBUG  
  printf (" in HierConditionData\n");
#endif

  /* If this is the first time that we've been called, let's do some setup stuff and check the input.*/
  if (mySearchParams->curHierLevel == 0) {
    INT4 maxDim, nextMaxDim, k;
        
#ifdef FCT_SO_DEBUG  
    printf(" FIRST TIME SET-UP \n");
    fprintf(stdout,"inout->numberSequences=%i\n",inout->numberSequences); 
    fflush( stdout );
    fprintf(stdout,"inout->sequences[0].name=%s\n",inout->sequences[0].name); 
    fflush( stdout );
#endif

    /* Check that there are the correct number of sequences. */
    if ( inout->numberSequences != numSequences ) {
      ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
    }
    
    /* Check that the data sequences are in the Correct Order. */
    if ((strstr( inout->sequences[dmroIndex].name, INPUTNAME_IFODMRO)  == 0) || \
	(strstr( inout->sequences[specIndex].name, INPUTNAME_SPECTRUM) == 0) || \
	(strstr( inout->sequences[respIndex].name, INPUTNAME_RESPONSE) == 0)) 
      {
	ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
      }/* end for */
    
    /* Check that the input data length is correct. */

    /* why do they have to be the same? -Linqing */
    /*    if (mySearchParams->inputDataLength != inout->sequences[dmroIndex].range.dTime.numberSamples) {
#ifdef FCT_SO_DEBUG  
    fprintf(stdout,"mySearchParams->inputDataLength=%i\n",mySearchParams->inputDataLength);
    fflush( stdout );
    fprintf(stdout,"inout->sequences[dmroIndex].range.dTime.numberSamples=%lli\n",inout->sequences[dmroIndex].range.dTime.numberSamples); 
    fflush( stdout );
#endif
      ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
    }
    */
    mySearchParams->inputDataStartGPS.gpsSeconds = inout->sequences[dmroIndex].range.dTime.startSec;
    mySearchParams->inputDataStartGPS.gpsNanoSeconds = inout->sequences[dmroIndex].range.dTime.startNan;
    mySearchParams->inputDataStart = inout->sequences[dmroIndex].range.dTime.startSec + inout->sequences[dmroIndex].range.dTime.startNan/1000000.0;
    mySearchParams->inputDataStop = inout->sequences[dmroIndex].range.dTime.stopSec + inout->sequences[dmroIndex].range.dTime.stopNan/1000000.0;
    dt=mySearchParams->fctInParams->dt=inout->sequences[dmroIndex].range.dTime.timeStepSize; /* make it local for now */
    mySearchParams->inputDataStepSize = dt;

    /*   mySearchParams->hierLevelUnits[0].range[0] = mySearchParams->inputDataStop - mySearchParams->inputDataStart; */
    mySearchParams->hierLevelUnits[0].range[0] = dt*mySearchParams->inputDataLength;
    /* For each level, set the time index range and the time step (in seconds) */
    for(j = 0; j < mySearchParams->numHierLevels; j++) {
      UINT4 l = 1;
      /* Nc = Fc * T */
      mySearchParams->hierLevelUnits[j].indexRange[0] = (INT4) (mySearchParams->hierLevelUnits[j].freqCutOff * mySearchParams->hierLevelUnits[j].range[0]); 
      /* delta t = 1 / (2 * Fc) */
      mySearchParams->hierLevelUnits[j].physicalStepSize[0] =  1.0/(2.0 * mySearchParams->hierLevelUnits[j].freqCutOff);
      while ( l < mySearchParams->hierLevelUnits[j].indexRange[0]/2 )
	l *= 2;
      mySearchParams->hierLevelUnits[j].indexRange[0] = l;
      mySearchParams->hierLevelUnits[j].freqCutOff = l / mySearchParams->hierLevelUnits[j].range[0];

      /* Also, make sure that for all but the first hierarchical
         level, the physical time range is never more than half the
         input data length (or else we may end up trying to search
         times where we have no data). */
      if ( j > 0 )
	if ( mySearchParams->hierLevelUnits[j].range[0] >= 0.5*mySearchParams->inputDataStepSize*mySearchParams->inputDataLength ) {
	  ABORT( status, CONDITIONDATA_EDATZ, CONDITIONDATA_MSGEDATZ );
	}
    }
#ifdef FCT_SO_DEBUG
    fprintf(stdout,"Checkpoint 1: first-time set up\n" );
    fprintf(stdout,"mySearchParams->hierLevelUnits[0].indexRange[0] = %i\n",mySearchParams->hierLevelUnits[0].indexRange[0] );
    printf( "mySearchParams->hierLevelUnits[0].freqCutOff = %f\n",mySearchParams->hierLevelUnits[0].freqCutOff );
    printf( "mySearchParams->hierLevelUnits[0].range[0] = %f\n", mySearchParams->hierLevelUnits[0].range[0] );
    fflush( stdout );
#endif
    FindThresholdAndUnits(status->statusPtr, mySearchParams, &(inout->sequences[specIndex]));
    CHECKSTATUSPTR (status);

#ifdef FCT_SO_DEBUG  
    fprintf(stdout,"Checkpoint 2: before MatchFCTUnitsToPhysicalUnits\n" );
    fflush( stdout );
#endif
    MatchFCTUnitsToPhysicalUnits(status->statusPtr, mySearchParams);
    CHECKSTATUSPTR (status);

#ifdef FCT_SO_DEBUG  
    fprintf(stdout,"Checkpoint 3: before SetInitialEvents\n" );
    fflush( stdout );
#endif
    SetInitialEvents(status->statusPtr, mySearchParams);
    CHECKSTATUSPTR (status);

#ifdef FCT_SO_DEBUG  
    fprintf( stdout,
	     "# of chirp dimensions: %i\n"
	     "Ranges:", mySearchParams->fctInParams->dimension - 1 );
   
 for ( k = 1; k < mySearchParams->fctInParams->dimension; k++ )
      fprintf( stdout, " %10.3e",
	       mySearchParams->hierLevelUnits[0].range[k] );
     
    fprintf( stdout, "\nInitial event list:\n" );
    {
      INT4 nEvents = 0;
      FCTHierEventType *curEvent = mySearchParams->hierLevelEvents[0];
      while ( curEvent ) {
	for ( k = 1; k < mySearchParams->fctInParams->dimension; k++ )
	   fprintf( stdout, " %5.2f", curEvent->location[k]/mySearchParams->hierLevelUnits[0].range[k] );
	fprintf( stdout, "\n" );
	  
	nEvents++;
	curEvent = curEvent->nextEvent;
      }
 
      fprintf( stdout, "# of events: %i\n", nEvents );
 
    }
    fflush( stdout );
#endif

    /* Find the two dimensions with the largest number of indices */
    for (j = 0; j < mySearchParams->numHierLevels; j++) {
      maxDim = 1;
      nextMaxDim = 1;
      mySearchParams->hierLevelUnits[j].numOfRowsInVol = 1;
      
      mySearchParams->hierLevelUnits[j].reportCutDimension = 1;
      for (k = 1; k < mySearchParams->fctInParams->dimension; k++) {
	mySearchParams->hierLevelUnits[j].indexRange[k] = (INT4)( mySearchParams->hierLevelUnits[j].range[k] 
	  / mySearchParams->hierLevelUnits[j].fctUnits[k] ) + 1;
	mySearchParams->hierLevelUnits[j].numOfRowsInVol *= mySearchParams->hierLevelUnits[j].indexRange[k];
	if (mySearchParams->hierLevelUnits[j].indexRange[k] >
	    mySearchParams->hierLevelUnits[j].indexRange[maxDim]) {
	  nextMaxDim = maxDim; 
	  maxDim = k;
	} else if (mySearchParams->hierLevelUnits[j].indexRange[k] >
		   mySearchParams->hierLevelUnits[j].indexRange[nextMaxDim]) {
	  nextMaxDim = k;
	}

#ifdef FCT_SO_DEBUG
	printf("mySearchParams->hierLevelUnits[%i].range[%i] = %10.3e\n", j, k, mySearchParams->hierLevelUnits[j].range[k] );
	printf("mySearchParams->hierLevelUnits[%i].fctUnits[%i] = %10.3e\n", j, k, mySearchParams->hierLevelUnits[j].fctUnits[k] );
	printf("mySearchParams->hierLevelUnits[%i].indexRange[%i] = %i\n\n", j, k, mySearchParams->hierLevelUnits[j].indexRange[k] );
#endif
      }
      mySearchParams->hierLevelUnits[j].parallelCutDimension = maxDim;
      mySearchParams->hierLevelUnits[j].reportCutDimension = nextMaxDim;
    }

    startIndex = 0;
    /* a bug? -Linqing, should be -1 */
    stopIndex = mySearchParams->inputDataLength-1;
  } else {
    startIndex = (INT4)((mySearchParams->curEvent->location[0] - \
			 mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[0]/2.0 )
			/ mySearchParams->inputDataStepSize);
    stopIndex = startIndex + mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[0]/mySearchParams->inputDataStepSize;

#ifdef FCT_SO_DEBUG
    printf("location[0]=%f, range[0]=%f, dataStart=%f, stepSize=%f\n", mySearchParams->curEvent->location[0],
	   mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[0], mySearchParams->inputDataStart,
	   mySearchParams->inputDataStepSize );
    printf("startIndex=%i, stopIndex=%i\n", startIndex, stopIndex );
#endif
  }  

  if (startIndex < 0 || stopIndex < 0 || startIndex >= stopIndex) {
    ABORT(status, CONDITIONDATA_ERANGE, CONDITIONDATA_MSGERANGE);
  }

  timeDataSliceLength = stopIndex - startIndex;


  dummyI2   = inout->sequences[dmroIndex].data.real4; /* I think that these should have better names. */
  dummyR4 = (REAL4 *)LALMalloc( mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]*sizeof(REAL4) );
  for ( j = 0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; j++ )
    dummyR4[j] = inout->sequences[specIndex].data.real4[j];
  dummyC8   = inout->sequences[respIndex].data.complex8;
  
  /* Setup the time location of this event. (For the first level we're setting up the time location for all events.) */
  mySearchParams->fctInParams->epoch.gpsSeconds = floor(startIndex*mySearchParams->inputDataStepSize);
  mySearchParams->fctInParams->epoch.gpsNanoSeconds = 1000000.0 * (startIndex*mySearchParams->inputDataStepSize - mySearchParams->fctInParams->epoch.gpsSeconds);
  mySearchParams->fctInParams->maxF=(INT4) mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].freqCutOff;


#ifdef FCT_SO_DEBUG  
  printf("epoch is %i,+%i, maxF=%i\n",mySearchParams->fctInParams->epoch.gpsSeconds,mySearchParams->fctInParams->epoch.gpsNanoSeconds,mySearchParams->fctInParams->maxF);
#endif
 
  /*   mySearchParams->fctInParams->dataLength/=2; */
   /*
  if ( mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0] != 2*mySearchParams->fctInParams->dataLength) {
   */
  changeFCTPlan(status->statusPtr, mySearchParams, mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]);
  CHECKSTATUSPTR (status);
    /*}*/


  
#ifdef FCT_SO_DEBUG
  printf("timeDataSliceLength = %i\n",timeDataSliceLength);
#endif
  
  tmpIn=NULL;
  LALCCreateVector(status->statusPtr, &tmpIn, timeDataSliceLength);
  CHECKSTATUSPTR (status);
  
    
  tmpOut=NULL;
  LALCCreateVector(status->statusPtr, &tmpOut, timeDataSliceLength);
  CHECKSTATUSPTR (status);

  
  /*  do DFT here , a simple one */
  
  for (j = 0; j <timeDataSliceLength; ++j)
  {
      tmpIn->data[j].re = (REAL4) dummyI2[startIndex + j];
      tmpIn->data[j].im = 0.0;
  }  
#ifdef FCT_SO_DEBUG
  printf("entering Linqing's ConditionData\n");
#endif
      
    
      
   /* normalization s.t. h_bal=0 */
    LALFCTMoment(status->statusPtr, timeDataSliceLength, tmpIn->data, &freqMoment);
    CHECKSTATUSPTR (status);
    
#ifdef FCT_SO_DEBUG
    printf("data mean, variance are %e, and %e\n",freqMoment.avg.re,freqMoment.sigma2.re);
#endif    
      
    /* normalize input to have h_bar=0, sigma_h=1 */
    for (j = 0; j <timeDataSliceLength; ++j)
    {
      tmpIn->data[j].re -=freqMoment.avg.re;
      /*       tmpIn->data[j].re /=sqrt(freqMoment.sigma2.re); */
	
#ifdef FCT_SO_DEBUG_DATA
      printf("data are %g\n",(REAL4) tmpIn->data[j].re);
#endif

    }

/*  Plan for doing the FFT on  the tseries */
  
  plan=NULL;
  /*  equavilent to fwd FFT in FFTW  with  -1 in the exponents */
  LALCreateForwardComplexFFTPlan(status->statusPtr, &plan, tmpIn->length, 0);
  CHECKSTATUSPTR (status);
  

  LALCOMPLEX8VectorFFT(status->statusPtr, tmpOut, tmpIn, plan);
  CHECKSTATUSPTR (status);


#ifdef FCT_SO_DEBUG
  /*  printf("mySearchParams->fctInParams->fctInput = %i\n",(int)mySearchParams->fctInParams->fctInput); */
  printf("after FFT\n");
#endif
  /* need it.  check w/ Jeff, -Linqing
  dataLength=timeDataSliceLength/2.;
  mySearchParams->fctInParams->fctInput->length=dataLength;

  /* put the frequency series in fctInput --- 
     a result of an unfortunate interracial marriage */
    for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
  {
    mySearchParams->fctInParams->fctInput->data[j].re =tmpOut->data[j].re;
    mySearchParams->fctInParams->fctInput->data[j].im =tmpOut->data[j].im;
  } 

    LALCDestroyVector(status->statusPtr, &tmpOut);
    CHECKSTATUSPTR (status);
  

    
  


  
    /* partially initialized in InitSearch */
    dt=mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[0];
    mySearchParams->fctInParams->distNorm=RsSun*sqrt(5./96.)/pow(LAL_PI,2./3.)*(1.0+cos(incl));
    mySearchParams->fctInParams->distNorm *= (pow(dt/Ts,1./6.)/mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]);
    /* debug */
  
#ifdef FCT_SO_DEBUG_DATA
  for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
  {
    printf(" FFT output  h(f)=%g+i %g\n", mySearchParams->fctInParams->fctInput->data[j].re,mySearchParams->fctInParams->fctInput->data[j].im);
  }
  
#endif
  
  /* check, 1/N normalization, need it because FFTW are not normalized  */
  
  for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
  {
    mySearchParams->fctInParams->fctInput->data[j].re /= mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0];
    mySearchParams->fctInParams->fctInput->data[j].im /= mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0];
  } 
  
    
  
#ifdef FCT_SO_DEBUG
  /*
  printf("after FFT, input mean.var\n");
  LALFCTMoment(status->statusPtr, dataLength, mySearchParams->fctInParams->fctInput->data, mySearchParams->fctInParams->freqMoment);
  printf("After FFT,  input mean and variance are %e, %e,%e %e\n",mySearchParams->fctInParams->freqMoment->avg.re,mySearchParams->fctInParams->freqMoment->avg.im,mySearchParams->fctInParams->freqMoment->sigma2.re,mySearchParams->fctInParams->freqMoment->sigma2.im);
  */  
#endif
  
  /* incorporate the response and noise power in */
  
  /*
    [h_real(f)*resp (f)]_input * (conjugate)(Resp (f) ) ;  if resp is n shape, conjugate
  */
  
  if (specIndex !=0 && respIndex !=0)
  {
      
    for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
    {
      tmpIn->data[j].re  =mySearchParams->fctInParams->fctInput->data[j].re*dummyC8[j].re -respShape*mySearchParams->fctInParams->fctInput->data[j].im*dummyC8[j].im;
      tmpIn->data[j].im  =mySearchParams->fctInParams->fctInput->data[j].re*dummyC8[j].im+respShape*mySearchParams->fctInParams->fctInput->data[j].im*dummyC8[j].re;
    } 
    
    
    /* deal with noise power which has a one-sided  PSD*/
    
    /*  
	If resp has U shape as in Teviet's code, 
	
	calculate h(f)/S(f)=
	([h(f)/ resp (f)]_input*resp (f)) /{[S_real(f)/|Resp|^2]_input * |Resp|^2} 
	={h_input *resp}_done /(S(f)*|resp|^2).
	
	
	First, calculate S'(f)=(S(f) *|resp|^2)  
    */
    
    distNorm=0.0;  /* just to make sure */
    if (respShape ==1)
    {
    
      for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j) {
	REAL4 temp1;
	dummyR4[j] /=fac; /* to avoid out of range at the next step */
	temp=dummyR4[j];

	if ( fabs( dummyC8[j].re ) > sqrt( 0.4*LAL_REAL4_MAX/temp ) )
	  dummyR4[j] = 0.0;
	else if ( fabs( dummyC8[j].im ) > sqrt( 0.4*LAL_REAL4_MAX/temp ) )
	  dummyR4[j] = 0.0;
	else {
	  temp1 = (temp * dummyC8[j].re) * dummyC8[j].re;
	  temp1 += (temp * dummyC8[j].im) * dummyC8[j].im;
	  dummyR4[j] = 1.0 / temp1;
	}
      }

      /* calulate part of distNorm */

      /* Calculate sum (j/N0)^(-7/6)/(S(f)*||resp||^2) if resp is U shaped */

      for (j =1; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
      {
	distNorm += (1./pow((1.0*j)/mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0],powInd)*dummyR4[j])/Mpc;
	/*	printf( "(j/range)^powInd = %10.3e, dummyR4[%i]=%10.3e, distNorm = %10.3e\n", pow((1.0*j)/mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0],powInd), j, dummyR4[j], distNorm );*/
      }

      mySearchParams->fctInParams->distNorm *= distNorm;
    }/* End if U shape  */
    else 
    {
      for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
      {
	dummyR4[j] = 1.0/dummyR4[j];
	distNorm += ((dummyC8[j].re/Mpc *dummyC8[j].re+dummyC8[j].im/Mpc *dummyC8[j].im)
		     /pow((1.0 *j)/mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0],powInd))*dummyR4[j];
      }
      
     mySearchParams->fctInParams->distNorm *= distNorm;

    }/* endif n shape */

    /* Calculate h'(f)/S'(f) for resp with either shape */

    for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
    {
      mySearchParams->fctInParams->fctInput->data[j].re = tmpIn->data[j].re*dummyR4[j];
      mySearchParams->fctInParams->fctInput->data[j].im = tmpIn->data[j].im*dummyR4[j];
    } 



#ifdef FCT_SO_DEBUG_DATA

    for (j=0; j<mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; j++)
    {
      printf("after resp/spec,  input h=%e+i %e, resp=%e+i%e, spect=%e\n", (mySearchParams->fctInParams->fctInput->data[j].re),(mySearchParams->fctInParams->fctInput->data[j].im), ((dummyC8[j].re)),((dummyC8[j].im)), dummyR4[j]);
      printf("fctInput are %g, %g\n",mySearchParams->fctInParams->fctInput->data[j].re,mySearchParams->fctInParams->fctInput->data[j].im);
    }
#endif


  }/* endif with noise spectrum and response */

#ifdef FCT_SO_DEBUG_DATA
    for (j =0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
    {
      printf(" input h(mid)(f)=%g+i %g\n", mySearchParams->fctInParams->fctInput->data[j].re,mySearchParams->fctInParams->fctInput->data[j].im);
    }
    
#endif

/* calculate the variance of the FCTInput and used it later at BuildEvent */
    /** debug **/
        LALFCTMoment(status->statusPtr, mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0], mySearchParams->fctInParams->fctInput->data, mySearchParams->fctInParams->freqMoment);
	CHECKSTATUSPTR (status);
    
#ifdef FCT_SO_DEBUG
       printf("After resp/noise,  input mean and variance are %e, %e,%e %e\n",(mySearchParams->fctInParams->freqMoment->avg.re),(mySearchParams->fctInParams->freqMoment->avg.im),(mySearchParams->fctInParams->freqMoment->sigma2.re),(mySearchParams->fctInParams->freqMoment->sigma2.im));
     
#endif

    /* check debug */
        mySearchParams->fctInParams->distNorm *= sqrt(2./mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]*(1./mySearchParams->fctInParams->freqMoment->sigma2.re+1./mySearchParams->fctInParams->freqMoment->sigma2.im)); 
       
#ifdef FCT_SO_DEBUG
      printf("end of  Linqing's ConditionData\n");
#endif

    /*   end of major new insertion from Linqing */ 


  for (j = 0; j < mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0]; ++j)
    {
      REAL4 cutoff = cos( LAL_PI_2*( (REAL4)j )/( (REAL4)mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].indexRange[0] ) );
      cutoff *= cutoff;
      mySearchParams->fctInParams->fctInput->data[j].re *=cutoff;
      mySearchParams->fctInParams->fctInput->data[j].im *=cutoff;
    }
  
  /* Place freq mask somewhere around here. */
  
  
  LALCDestroyVector(status->statusPtr, &tmpIn);
  CHECKSTATUSPTR (status);
  
  LALDestroyComplexFFTPlan(status->statusPtr, &plan);
  CHECKSTATUSPTR (status);

  LALFree( dummyR4 );

  mySearchParams->curEventConditioned = 1;
  
  DETATCHSTATUSPTR (status);        
  RETURN( status ); 
}


void changeFCTPlan(LALStatus *status, FCTSearchParams * mySearchParams, INT4 dataLength) {
  extern LALFCTPhaseFn phaseFuncArray[];
  /* These are all structures that are used as input into the FCT code routines */
  FCTInStrucType    fctInStruct;  
  LALCreateFCTPlanInput FCTPlanIn;
  LALFCTSetUnitsInput setUnitsIn; 
  INT4 j;
  
  INITSTATUS( status, "changeFCTPlan", FCTCONDITIONDATAC ); 
  ATTATCHSTATUSPTR (status);
  
#ifdef FCT_SO_DEBUG
  printf("In ChangeFCTPlan\n");
#endif

  /* Destory the old plan and inputdata. */ 
  if (mySearchParams->fctInParams->fctInput) {
#ifdef FCT_SO_DEBUG
  printf("Destroying old data.\n");
#endif
    LALCDestroyVector(status->statusPtr, &(mySearchParams->fctInParams->fctInput));
    CHECKSTATUSPTR (status);

  }
  
  if (mySearchParams->fctInParams->fctPlan) {
#ifdef FCT_SO_DEBUG
  printf("Destroying old plan.\n");
#endif
    LALDestroyFCTPlan(status->statusPtr, &(mySearchParams->fctInParams->fctPlan));
    CHECKSTATUSPTR (status);
  }

  /*  dataLength/=2;*/
  mySearchParams->fctInParams->dataLength = dataLength;


  FCTPlanIn.data_length = dataLength;
  FCTPlanIn.number_of_dimensions = mySearchParams->fctInParams->dimension; /*  including dim 0  */
  FCTPlanIn.dimension_0_stride = 1;

#ifdef FCT_SO_DEBUG
  /* phaseFuncArray[j]  is a pointer, -Linqing */
  /* 
  printf("Setting hase function array:\n");
  for (j=0;j<mySearchParams->fctInParams->dimension-1;j++)
    {
    printf( "%i ", (INT4)phaseFuncArray[j] );
    }
    printf( "\n%i\n", (INT4)FCTPlanIn.phase_fn );
    
  */
#endif
  for (j=0;j<mySearchParams->fctInParams->dimension-1;j++)
    {
      FCTPlanIn.phase_fn[j] = phaseFuncArray[j];
    }
#ifdef FCT_SO_DEBUG
  printf("Phase function array set.\n");
#endif
    
  fctInStruct.planIn=FCTPlanIn;

  mySearchParams->fctInParams->cube->stride[0]=FCTPlanIn.dimension_0_stride; 
  mySearchParams->fctInParams->cube->start_locations[0]=0;
  mySearchParams->fctInParams->cube->end_locations[0]=dataLength; 


  /*  Define offset and as a constant there  */
  
  setUnitsIn.offset=0.0;
  setUnitsIn.delta=1.0/dataLength; /* Do we need to change this? */

  fctInStruct.setUnitsIn=setUnitsIn;
  fctInStruct.maxFFTSeg=10;  /*  Check, as part of input? */

  /* you can change it, but it has to be assigned , -Linqing */
  mySearchParams->fctInParams->numDataCubes=1;  

  mySearchParams->fctInParams->fctInput=NULL; /* necessary for LALCCreateVector() */
  LALCCreateVector(status->statusPtr, &(mySearchParams->fctInParams->fctInput),dataLength);
  CHECKSTATUSPTR (status);
#ifdef FCT_SO_DEBUG
  printf("mySearchParams->fctInParams->fctInput = %i\n",(int)mySearchParams->fctInParams->fctInput);
  printf("FCTPlanIn.data_length = %i\n", FCTPlanIn.data_length);
#endif
  
  mySearchParams->fctInParams->fctPlan=NULL;
  LALMyFCTInitPlan(status->statusPtr, &fctInStruct, &(mySearchParams->fctInParams->fctPlan));
  CHECKSTATUSPTR (status);
  
  if(!mySearchParams->fctInParams->fctPlan)
    {
      ABORT( status, CONDITIONDATA_ENULL, CONDITIONDATA_MSGENULL);
    }
  
  
#ifdef FCT_SO_DEBUG
  printf("out of ChangeFCTPlan\n");
#endif

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
} 


  
  /* Cube stride stuff */
  
/*    for (j=1; j<mySearchParams->fctInParams->dimension; j++) */
/*      { */
/*        mySearchParams->fctInParams->cube->stride[j]=stride[j]; */
/*      } */
  
/*    mySearchParams->fctInParams->numDataCube=1;  
      mySearchParams->fctInParams->cube->stride[0]=planIn.dimension_0_stride; 
      mySearchParams->fctInParams->cube->start_locations[0]=0;
      mySearchParams->fctInParams->cube->end_locations[0]=dataLength/2;  */
/* due to some bizarre thing in fct.c */ 




