/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeApplySearchMasterC"> ********
Author: Daw, E
$Id: SlopeApplySearchMaster.c,v 1.2 2002/06/26 18:40:55 edaw Exp $
******** </lalVerbatim> ********/

#ifdef DEBUG_SLOPE
#include <stdio.h>
#endif
#include <math.h>

/* lal standard library */
#include <lal/LALStdlib.h>

/* lal MPI interface */
#include <lal/Comm.h>

/* lal slope filters */
#include <lal/SlopeDetectorFilter.h>

/* structures and definitions for this code segment */
#include <SlopeDatatypes.h>
#include <SlopeAddTimeToGpsSecNan.h>
#include <SlopeAddEventToLinkedList.h>
#include <SlopeApplySearchMaster.h>

/* this is a temporary fix to stop LDAS timing my job out */
/* when writing large number of events to the database from */
/* a single job. The cap will be removed and replaced by a */
/* reentrant LALApplySearch() that igor has written after testing */
/* following S1 */
#define SLOPE_MAX_EVENTS_PER_JOB 3000

NRCSID( SLOPEAPPLYSEARCHMASTERC, "$Id: ");

void
SlopeApplySearchMaster(LALStatus*          status,
		       LALSearchOutput*    output,
		       SlopeSearchParams*  params) 
{
  REAL4Vector dataIn;
  REAL4Vector dataSendOut;
  REAL4* dataOut;
  UINT4 dataSearchCount = 0;
  UINT4 nSearch = 0;
  UINT4 nDataOut = 0;
  UINT4 peakBin = 0;
  UINT4 peakTimeSec;
  UINT4 peakTimeNan;
  REAL4 peakAmplitude = 0.0;
  REAL4 secAfterStartStretch;
  REAL4 searchidentifier;
  SLOPEFilterParams* filterp;
  SLOPEFilterParams filterp_local;
  UINT4 hcount;
#ifdef SLDEBUG
  UINT4 i;
#endif  

#ifdef SLDEBUG
  printf("Starting apply search master.\n");
#endif

  INITSTATUS( status, "SlopeApplySearchMaster", SLOPEAPPLYSEARCHMASTERC );
  ATTATCHSTATUSPTR(status);

  /* set pointer for filter parameters substructure of */
  /* the search parameters array */
  filterp = (SLOPEFilterParams*)params;

  /* get the length of the data output array */
  nDataOut = params->nData - params->forder + 1 ;
#ifdef SLDEBUG
  printf("number of bins in output array: %u\n",nDataOut);
#endif

  /* get the number of bins to search */
  nSearch = params->nData - 
    params->forder -
    params->peakWidth + 2;

  /* construct data vector for input data array. */
  dataIn.length = params->nData;
  dataIn.data = (REAL4*)params->dataNew;

  /* allocate memory for output data */
  dataOut = LALCalloc( nDataOut , sizeof(REAL4) );
  
  /* make structure containing output data array and size */
  dataSendOut.length = nDataOut;
  dataSendOut.data = dataOut;

#ifdef SLDEBUG
  printf("Starting filtration\n");
#endif

  /* set the frequency equal to the peak identifier */
  /* so I can identify which search turned up a trigger */
  /* this is a temporary kluge for E7 */
  searchidentifier = (REAL4)params->function_select;

  switch( params->function_select ) {
  case FILTER_OUTPUT_PRELIM:
    /* filter the data using the 'old' slope detector filter */
    LALSlopeDetectorFilter(status->statusPtr,
			   &dataSendOut,
			   &dataIn,
			   params->forder);
    break;
  case FILTER_OUTPUT_SLOPE:
    LALSlopeLineFitFilter(status->statusPtr,
			  &dataSendOut,
			  &dataIn,
			  *(filterp));
    break;
  case FILTER_OUTPUT_OFFSET:
    LALSlopeLineFitFilter(status->statusPtr,
			  &dataSendOut,
			  &dataIn,
			  *(filterp));
    break;
  case FILTER_OUTPUT_ALF:
    LALSlopeLineFitFilter(status->statusPtr,
			  &dataSendOut,
			  &dataIn,
			  *(filterp));
    break;
  case FILTER_OUTPUT_TAPS_SET_BOXCAR:
    if((*(filterp->taps_set)) == 0) {
      LALSlopeConvolutionFilter(status->statusPtr,
				&dataSendOut,
				&dataIn,
				*(filterp));
    }
    filterp_local.forder = filterp->forder;
    filterp_local.tap = filterp->tap;
    filterp_local.history_allocated = filterp->history_allocated;
    filterp_local.taps_set = filterp->taps_set;
    filterp_local.history = filterp->history;
    filterp_local.function_select = FILTER_OUTPUT_CONVOLVE;
    filterp_local.waveform_offset = filterp->waveform_offset;
    filterp_local.sampling_period_s = filterp->sampling_period_s;
    LALSlopeConvolutionFilter(status->statusPtr,
			      &dataSendOut,
			      &dataIn,
			      filterp_local);
    break;
  case FILTER_OUTPUT_TAPS_SET_GAUSSIAN:
    if((*(filterp->taps_set)) == 0) {
      LALSlopeConvolutionFilter(status->statusPtr,
				&dataSendOut,
				&dataIn,
				*(filterp));
    }
    filterp_local.forder = filterp->forder;
    filterp_local.tap = filterp->tap;
    filterp_local.history_allocated = filterp->history_allocated;
    filterp_local.taps_set = filterp->taps_set;
    filterp_local.history = filterp->history;
    filterp_local.function_select = FILTER_OUTPUT_CONVOLVE;
    filterp_local.waveform_offset = filterp->waveform_offset;
    filterp_local.sampling_period_s = filterp->sampling_period_s;
    LALSlopeConvolutionFilter(status->statusPtr,
			      &dataSendOut,
			      &dataIn,
			      filterp_local);
    break;
  case FILTER_OUTPUT_TAPS_SET_SINE:
    if((*(filterp->taps_set)) == 0) {
      LALSlopeConvolutionFilter(status->statusPtr,
				&dataSendOut,
				&dataIn,
				*(filterp));
    }
    filterp_local.forder = filterp->forder;
    filterp_local.tap = filterp->tap;
    filterp_local.history_allocated = filterp->history_allocated;
    filterp_local.taps_set = filterp->taps_set;
    filterp_local.history = filterp->history;
    filterp_local.function_select = FILTER_OUTPUT_CONVOLVE;
    filterp_local.waveform_offset = filterp->waveform_offset;
    filterp_local.sampling_period_s = filterp->sampling_period_s;
    LALSlopeConvolutionFilter(status->statusPtr,
			      &dataSendOut,
			      &dataIn,
			      filterp_local);
    break;
  default:
      ABORT(status, APPLYSEARCH_NOSUCHFILTER, APPLYSEARCH_NOSUCHFILTERMESG );
  }

#ifdef SLDEBUG
  printf("Starting peak search.\n");
#endif

  /* write the latter end of the output data to the output history buffer */
  for(hcount=0 ; hcount < (params->peakWidth - 1) ; ++hcount) {
    params->peakHistBuf[hcount] = dataOut[nSearch + hcount];
  }

  /* do a peak search on the filtered data */
  dataSearchCount = 0;
  while(dataSearchCount < nSearch) {

#ifdef SLDEBUG
    /* print out search information */
    printf("%u\t%f\t%f\t%f\n",dataSearchCount,
	   dataIn.data[dataSearchCount],
	   dataOut[dataSearchCount],
	   params->triggerThreshold);
#endif

    /* test if the next bin exceeds the threshold */
    if( dataOut[dataSearchCount] > params->triggerThreshold ) {

      /* if first trigger, allocate memory to first element in */
      /* linked list */
      /*if( params->numberOfTriggersSoFar == 0 ) {
	params->currentEvent = LALMalloc( sizeof(SlopeDetections) );
	params->firstEvent = params->currentEvent;
	}*/

      /* increment counter for detected events */
      ++params->numberOfTriggersSoFar;

      /* set event amplitude */
      peakAmplitude = dataOut[dataSearchCount];

      /* set bin where peak starts */
      peakBin = dataSearchCount;

      /* look over the (peakWidth) bins after the one exceeding */
      /* the threshold. The peak is positioned at the tallest one */
      while(dataSearchCount < ( peakBin + 
			     params->peakWidth - 1 )) {
	++dataSearchCount;
	if( dataOut[dataSearchCount] > peakAmplitude ) {
	  peakAmplitude = dataOut[dataSearchCount];
	  peakBin = dataSearchCount;
	}
      }

      /* get peak time */
      secAfterStartStretch = (REAL4)peakBin/((REAL4)params->samplingRate);
      peakTimeSec = params->startTimeSec;
      peakTimeNan = params->startTimeNan;
      SlopeAddTimeToGpsSecNan(&peakTimeSec, 
			      &peakTimeNan, secAfterStartStretch);

#ifdef SLDEBUG
      printf("Adding event at bin %u\n",peakBin); 
#endif

      /* write newly located peak to linked list */

      if ( params->numberOfTriggersSoFar <= SLOPE_MAX_EVENTS_PER_JOB ) {
	
	SlopeAddEventToLinkedList(status->statusPtr,
				  peakTimeSec,
				  peakTimeNan,
				  peakAmplitude,
				  ((REAL4)params->peakWidth / 
				   (REAL4)params->samplingRate),
				  searchidentifier,
				  params->samplingRate / 2,
				  &(params->currentEvent),
				  &(params->firstEvent) );
	
#ifdef SLDEBUG
	printf("Added event. Data search count is %u / %u\n",
	       dataSearchCount,dataSendOut.length);
	/* print out search information */
	printf("next data in %f\n",
	       dataIn.data[dataSearchCount]);
	printf("next data out %f\n",dataOut[dataSearchCount]);
#endif
	
      }


    } else {
      ++dataSearchCount;
    }

    /* update progress in search */
    output->fracRemaining = ( (REAL4)(nSearch - dataSearchCount) / nSearch );
  }

#ifdef SLDEBUG
  printf("Finished search.\n");
#endif

  LALFree(dataOut);

#ifdef SLDEBUG
  printf("Freed dataOut.\n");
#endif

  DETATCHSTATUSPTR(status);
  RETURN(status);
}






